"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var PassportStrategy = require('passport-strategy');

var jwt = require('jsonwebtoken');

var _require = require('util'),
    promisify = _require.promisify;

var lookup = require('./lookup');

var MagicLinkStrategy =
/*#__PURE__*/
function (_PassportStrategy) {
  (0, _inherits2.default)(MagicLinkStrategy, _PassportStrategy);

  /**
   * @param {string} secret - The secret to sign the token
   * @param {Array} userFields - An array of mandatory user fields from the request
   * @param {Array} tokenField - The token field from the request
   * @param {Number} ttl - Time to live
   * @param {Boolean} passReqToCallbacks - Whether the req should be passed do the sendToken callback
   * @param {Boolean} verifyUserAfterToken - Whether the user should be verified after the token verification
   * @param {Function} sendToken - A function to deliver the token
   * @param {Function} verifyUser - A function to verify the user
   */
  function MagicLinkStrategy(_ref, sendToken, verifyUser) {
    var _this;

    var secret = _ref.secret,
        userFields = _ref.userFields,
        tokenField = _ref.tokenField,
        _ref$ttl = _ref.ttl,
        ttl = _ref$ttl === void 0 ? 60 * 10 : _ref$ttl,
        _ref$passReqToCallbac = _ref.passReqToCallbacks,
        passReqToCallbacks = _ref$passReqToCallbac === void 0 ? false : _ref$passReqToCallbac,
        _ref$verifyUserAfterT = _ref.verifyUserAfterToken,
        verifyUserAfterToken = _ref$verifyUserAfterT === void 0 ? false : _ref$verifyUserAfterT;
    (0, _classCallCheck2.default)(this, MagicLinkStrategy);
    if (!secret) throw new Error('Magic Link authentication strategy requires an encryption secret');
    if (!userFields || !userFields.length) throw new Error('Magic Link authentication strategy requires an array of mandatory user fields');
    if (!tokenField) throw new Error('Magic Link authentication strategy requires a token field');
    if (!sendToken) throw new Error('Magic Link authentication strategy requires a sendToken function');
    if (!verifyUser) throw new Error('Magic Link authentication strategy requires a verifyUser function');
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(MagicLinkStrategy).call(this));
    _this.name = 'magiclink';
    _this.secret = secret;
    _this.ttl = ttl;
    _this.passReqToCallbacks = passReqToCallbacks;
    _this.verifyUserAfterToken = verifyUserAfterToken;
    _this.userFields = userFields;
    _this.tokenField = tokenField;
    _this.sendToken = sendToken;
    _this.verifyUser = verifyUser;
    return _this;
  }

  (0, _createClass2.default)(MagicLinkStrategy, [{
    key: "authenticate",
    value: function () {
      var _authenticate = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee(req) {
        var options,
            sanitizedOptions,
            _args = arguments;
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                options = _args.length > 1 && _args[1] !== undefined ? _args[1] : {};
                sanitizedOptions = (0, _objectSpread2.default)({
                  action: 'acceptToken'
                }, options); // Request token logic
                // ====================================

                if (!(sanitizedOptions.action === 'requestToken')) {
                  _context.next = 4;
                  break;
                }

                return _context.abrupt("return", this.requestToken(req, sanitizedOptions));

              case 4:
                if (!(sanitizedOptions.action === 'acceptToken')) {
                  _context.next = 6;
                  break;
                }

                return _context.abrupt("return", this.acceptToken(req, sanitizedOptions));

              case 6:
                return _context.abrupt("return", this.error(new Error('Unknown action')));

              case 7:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function authenticate(_x) {
        return _authenticate.apply(this, arguments);
      }

      return authenticate;
    }()
  }, {
    key: "requestToken",
    value: function () {
      var _requestToken = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee2(req, options) {
        var userFields, user, i, fieldValue, createToken, token;
        return _regenerator.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                userFields = {};
                i = 0;

              case 2:
                if (!(i < this.userFields.length)) {
                  _context2.next = 11;
                  break;
                }

                fieldValue = lookup(req.body, this.userFields[i]) || lookup(req.query, this.userFields[i]);

                if (fieldValue) {
                  _context2.next = 7;
                  break;
                }

                userFields = null;
                return _context2.abrupt("break", 11);

              case 7:
                userFields[this.userFields[i]] = fieldValue;

              case 8:
                i++;
                _context2.next = 2;
                break;

              case 11:
                if (userFields) {
                  _context2.next = 13;
                  break;
                }

                return _context2.abrupt("return", this.fail(new Error('Mandatory user fields missing')));

              case 13:
                if (this.verifyUserAfterToken) {
                  _context2.next = 33;
                  break;
                }

                _context2.prev = 14;

                if (!this.passReqToCallbacks) {
                  _context2.next = 21;
                  break;
                }

                _context2.next = 18;
                return this.verifyUser(req, userFields);

              case 18:
                user = _context2.sent;
                _context2.next = 24;
                break;

              case 21:
                _context2.next = 23;
                return this.verifyUser(userFields);

              case 23:
                user = _context2.sent;

              case 24:
                _context2.next = 29;
                break;

              case 26:
                _context2.prev = 26;
                _context2.t0 = _context2["catch"](14);
                return _context2.abrupt("return", this.error(_context2.t0));

              case 29:
                if (user) {
                  _context2.next = 31;
                  break;
                }

                return _context2.abrupt("return", this.fail({
                  message: options.authMessage || "No user found"
                }, 400));

              case 31:
                _context2.next = 34;
                break;

              case 33:
                user = userFields;

              case 34:
                // Generate JWT
                createToken = promisify(jwt.sign);
                _context2.prev = 35;
                _context2.next = 38;
                return createToken({
                  user: user,
                  iat: Math.floor(Date.now() / 1000)
                }, this.secret, {
                  expiresIn: this.ttl
                });

              case 38:
                token = _context2.sent;
                _context2.next = 44;
                break;

              case 41:
                _context2.prev = 41;
                _context2.t1 = _context2["catch"](35);
                return _context2.abrupt("return", this.error(_context2.t1));

              case 44:
                _context2.prev = 44;

                if (!this.passReqToCallbacks) {
                  _context2.next = 50;
                  break;
                }

                _context2.next = 48;
                return this.sendToken(req, user, token);

              case 48:
                _context2.next = 52;
                break;

              case 50:
                _context2.next = 52;
                return this.sendToken(user, token);

              case 52:
                _context2.next = 57;
                break;

              case 54:
                _context2.prev = 54;
                _context2.t2 = _context2["catch"](44);
                return _context2.abrupt("return", this.error(_context2.t2));

              case 57:
                return _context2.abrupt("return", this.pass({
                  message: 'Token succesfully delivered'
                }));

              case 58:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[14, 26], [35, 41], [44, 54]]);
      }));

      function requestToken(_x2, _x3) {
        return _requestToken.apply(this, arguments);
      }

      return requestToken;
    }()
  }, {
    key: "acceptToken",
    value: function () {
      var _acceptToken = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee3(req, options) {
        var token, user, verifyToken, _ref2, tokenUser;

        return _regenerator.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                token = lookup(req.body, this.tokenField) || lookup(req.query, this.tokenField);

                if (token) {
                  _context3.next = 3;
                  break;
                }

                return _context3.abrupt("return", this.fail({
                  message: 'Token missing'
                }));

              case 3:
                _context3.prev = 3;
                // Verify token
                verifyToken = promisify(jwt.verify);
                _context3.next = 7;
                return verifyToken(token, this.secret);

              case 7:
                _ref2 = _context3.sent;
                tokenUser = _ref2.user;
                user = tokenUser;
                _context3.next = 15;
                break;

              case 12:
                _context3.prev = 12;
                _context3.t0 = _context3["catch"](3);
                return _context3.abrupt("return", this.fail({
                  message: _context3.t0.message
                }));

              case 15:
                if (!this.verifyUserAfterToken) {
                  _context3.next = 33;
                  break;
                }

                _context3.prev = 16;

                if (!this.passReqToCallbacks) {
                  _context3.next = 23;
                  break;
                }

                _context3.next = 20;
                return this.verifyUser(req, user);

              case 20:
                user = _context3.sent;
                _context3.next = 26;
                break;

              case 23:
                _context3.next = 25;
                return this.verifyUser(user);

              case 25:
                user = _context3.sent;

              case 26:
                _context3.next = 31;
                break;

              case 28:
                _context3.prev = 28;
                _context3.t1 = _context3["catch"](16);
                return _context3.abrupt("return", this.error(_context3.t1));

              case 31:
                if (user) {
                  _context3.next = 33;
                  break;
                }

                return _context3.abrupt("return", this.fail({
                  message: options.authMessage || "No user found"
                }, 400));

              case 33:
                return _context3.abrupt("return", this.success(user));

              case 34:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[3, 12], [16, 28]]);
      }));

      function acceptToken(_x4, _x5) {
        return _acceptToken.apply(this, arguments);
      }

      return acceptToken;
    }()
  }]);
  return MagicLinkStrategy;
}(PassportStrategy);

module.exports = MagicLinkStrategy;